#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Path planning evaluation script
@author: P.Cizek
"""
import sys

import map as mp
import planner as pl

import time
import numpy as np

import math

import eval_functions as ef

#settings
num_repetitions = 1
show_path = True

#instantiation of map and planner
map = mp.Map()
planner = pl.Planner()

################################################
# Testing
################################################
scenarios = (("./mazes/maze01.png",(210,210),(0,0),1,(5,105),(205,105),False), 
			("./mazes/maze02.png",(210,210),(0,0),1,(5,105),(205,105),False), 
			("./mazes/maze03.png",(210,210),(0,0),1,(5,105),(205,105),False), 
			("./mazes/maze04.png",(210,210),(0,0),1,(5,105),(205,105),False), 
			("./mazes/maze05.png",(210,210),(0,0),1,(5,105),(205,105),False), 
			("./mazes/maze06.png",(410,410),(0,0),1,(5,205),(405,205),False), 
			("./mazes/maze07.png",(410,410),(0,0),1,(5,205),(405,205),False), 
			("./mazes/maze08.png",(410,410),(0,0),1,(5,205),(405,205),False), 
			("./mazes/maze09.png",(410,410),(0,0),1,(5,205),(405,205),False), 
			("./mazes/maze10.png",(410,410),(0,0),1,(5,205),(405,205),False), 
			("./mazes/maze11.png",(610,610),(0,0),1,(5,305),(605,305),False), 
			("./mazes/maze12.png",(610,610),(0,0),1,(5,305),(605,305),False), 
			("./mazes/maze13.png",(610,610),(0,0),1,(5,305),(605,305),False), 
			("./mazes/maze14.png",(610,610),(0,0),1,(5,305),(605,305),False), 
			("./mazes/maze15.png",(610,610),(0,0),1,(5,305),(605,305),False), 
			("./mazes/maze16.png",(810,810),(0,0),1,(5,405),(805,405),False), 
			("./mazes/maze17.png",(810,810),(0,0),1,(5,405),(805,405),False), 
			("./mazes/maze18.png",(810,810),(0,0),1,(5,405),(805,405),False), 
			("./mazes/maze19.png",(810,810),(0,0),1,(5,405),(805,405),False), 
			("./mazes/maze20.png",(810,810),(0,0),1,(5,405),(805,405),False),
			("./mazes/maze1.png",(10,8.6),(5,4.3),0.1,(92,75),(5,20),True))

######################################
#perform individual tests
######################################
for test in scenarios:
	map.from_file(test[0],test[1],test[2],test[3])
	start = test[4]
	goal = test[5]
	execute = test[6]

	if show_path:
		#show sketch of the path
		mp.plot_path(map.get_grid(), (start, goal))

	cpu_time = []
	feasible_stat = []
	length_stat = []
	clearance_stat = []

	for i in range(0, num_repetitions):
		#start CPU time measurement
		tick = time.clock()
		
		######################################
		#path planning
		######################################
		path = planner.plan(map, start, goal)
		
		#stop CPU time measurement
		cpu_time.append(time.clock() - tick)
		
		#show the planned path
		if show_path:
			mp.plot_path(map.get_grid(), path)
		
		#evaluate the solution
		feasible, length, clearance = ef.eval_path(map, path, start, goal)
		feasible_stat.append(feasible)
		length_stat.append(length)
		clearance_stat.append(clearance)
		
		######################################
		#path execution
		######################################
		if execute:
			ret = planner.execute(map, start, goal)
	
	######################################
	#calculation of statistics
	######################################

	#feasible statistics
	feasible_cnt = np.sum(feasible_stat)

	#time statistics
	min_cpu_time = np.min(cpu_time)
	max_cpu_time = np.max(cpu_time)
	mean_cpu_time = np.mean(cpu_time)

	#length statistics
	min_length = np.min(length_stat)
	max_length = np.max(length_stat)
	mean_length = np.mean(length_stat)

	#clearance statistics
	min_clearance = np.min([x[0] for x in clearance_stat])
	max_clearance = np.max([x[1] for x in clearance_stat])
	mean_clearance = np.mean([x[2] for x in clearance_stat])

	######################################
	#statistics printing
	######################################
	print("Scenario" + str(test[0]))
	print("Feasible solutions: " + str(feasible_cnt) + "/" + str(num_repetitions))
	print("CPU time: min: " + str(min_cpu_time) + " max: " + str(max_cpu_time) + " mean: " + str(mean_cpu_time))
	print("Path length: min: " + str(min_length) + " max: " + str(max_length) + " mean: " + str(mean_length))
	print("Path clearance: min: " + str(min_clearance) + " max: " + str(max_clearance) + " mean: " + str(mean_clearance))
	print("")