# -*- coding: utf-8 -*-
"""
Evaluation script helper functions
@author: P.Cizek
"""
import sys
import numpy as np
import scipy.ndimage as ndimg

import math

import matplotlib.pyplot as plt

import map as mp
#from PIL import Image

################################################
# Helper functions
################################################
def euclidean_dist(a,b):
	(x1, y1) = a
	(x2, y2) = b
	return math.sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2))

def distance_transform(grid_map):
	#distance transform algorithm
	dt_map = ndimg.distance_transform_edt(grid_map)
	return dt_map

def bresenham_line_check(map, a, b):
	#Bresenham's line algorithm
	(x0, y0) = a
	(x1, y1) = b
	line = []
	dx = abs(x1 - x0)
	dy = abs(y1 - y0)
	x, y = x0, y0
	sx = -1 if x0 > x1 else 1
	sy = -1 if y0 > y1 else 1
	if dx > dy:
		err = dx / 2.0
		while x != x1:
			if not map.passable((x, y)): 
				return None
			line.append((x,y))
			err -= dy
			if err < 0:
				y += sy
				err += dx
			x += sx
	else:
		err = dy / 2.0
		while y != y1:
			if not map.passable((x, y)):
				 return None
			line.append((x,y))
			err -= dx
			if err < 0:
				x += sx
				err += dy
			y += sy
	if not map.passable((x, y)):
		return None
	else:
		line.append((x,y))
		return line

def eval_path(map, path, start, goal):
	#path evaluation function
	feasible = True
	length = 0
	clearance = [0, 0, 0]
	clearance_arr = []
	
	#calculate distance transform map
	dt_map = distance_transform(map.get_grid())
	
	for i in range(0,len(path)-1):
		#extract path points
		(x1, y1) = path[i]
		(x2, y2) = path[i+1]
		
		#compute distance between points
		length += euclidean_dist((x1, y1), (x2, y2))
		
		#test feasibility
		dpath = bresenham_line_check(map, (x1, y1), (x2, y2))
		if dpath == None:
			feasible = False
		else:
			#remove last point
			dpath = dpath[:-1]
			#calculate clearance of individual points
			for path_point in dpath:
				clearance_arr.append(dt_map[path_point[0], path_point[1]])
	if feasible:
		#calculate clearance array scores
		clearance[0] = np.min(clearance_arr)
		clearance[1] = np.max(clearance_arr)
		clearance[2] = np.sum(clearance_arr)
	
	return feasible, length, clearance